//Printer motor
//Source & info at: www.HomoFaciens.de/technics-base-circuits-encoder-disc_en_navion.htm
//Sensor disc with 2.2k, 560 and 0 Ohms

#include <util/delay.h>

int stepStatus = 0;


int dutyCycle = 0;
signed long setPoint = 0;
signed long MouseX = 0;
signed long MouseY = 0;

byte readByte = 0;
int sensorStatus = 0;
int sensorStatusOld = 0;
unsigned long millisStart = 0;
unsigned long millisNow = 0;
unsigned long PS2MillisStart;
char stat,x,y;

byte PS2ReadByte = 0;


#define MOTOR_DIRECTION 5
#define MOTOR_PWM 6

#define PS2CLOCK  7
#define PS2DATA   8


#define P_FRACTION 1.0     //Proportional factor of control loop 0.001 - 10.0 (1.0)
#define STEP_MARGIN 4L     //10 - 1000 (1)

#define MIN_DUTYCYCLE 200   //0 - 255 (125)
#define MAX_DUTYCYCLE 250  //0 - 255 (255)




void PS2GoHi(int pin){
  pinMode(pin, INPUT);
  digitalWrite(pin, HIGH);
}

void PS2GoLo(int pin){
  pinMode(pin, OUTPUT);
  digitalWrite(pin, LOW);
}

void PS2Write(unsigned char data){
  unsigned char parity=1;

  PS2GoHi(PS2DATA);
  PS2GoHi(PS2CLOCK);
  delayMicroseconds(300);
  PS2GoLo(PS2CLOCK);
  delayMicroseconds(300);
  PS2GoLo(PS2DATA);
  delayMicroseconds(10);
  PS2GoHi(PS2CLOCK);

  while(digitalRead(PS2CLOCK)==HIGH);

  for(int i=0; i<8; i++){
    if(data&0x01) PS2GoHi(PS2DATA);
    else PS2GoLo(PS2DATA);
    while(digitalRead(PS2CLOCK)==LOW);
    while(digitalRead(PS2CLOCK)==HIGH);
    parity^=(data&0x01);
    data=data>>1;
  }

  if(parity) PS2GoHi(PS2DATA);
  else PS2GoLo(PS2DATA);

  while(digitalRead(PS2CLOCK)==LOW);
  while(digitalRead(PS2CLOCK)==HIGH);

  PS2GoHi(PS2DATA);
  delayMicroseconds(50);

  while(digitalRead(PS2CLOCK)==HIGH);
  while((digitalRead(PS2CLOCK)==LOW)||(digitalRead(PS2DATA)==LOW));

  PS2GoLo(PS2CLOCK);
}

unsigned char PS2Read(void){
  unsigned char data=0, bit=1;

  PS2GoHi(PS2CLOCK);
  PS2GoHi(PS2DATA);
  delayMicroseconds(50);
  while(digitalRead(PS2CLOCK)==HIGH);

  delayMicroseconds(5);
  while(digitalRead(PS2CLOCK)==LOW);

  for(int i=0; i<8; i++){
    while(digitalRead(PS2CLOCK)==HIGH);
    if(digitalRead(PS2DATA)==HIGH) data|=bit;
    while(digitalRead(PS2CLOCK)==LOW);
    bit=bit<<1;
  }

  while(digitalRead(PS2CLOCK)==HIGH);
  while(digitalRead(PS2CLOCK)==LOW);
  while(digitalRead(PS2CLOCK)==HIGH);
  while(digitalRead(PS2CLOCK)==LOW);

  PS2GoLo(PS2CLOCK);

  return data;
}

void PS2MouseInit(void){
  PS2Write(0xFF);
  for(int i=0; i<3; i++) PS2Read();
  PS2Write(0xF0);
  PS2Read();
  delayMicroseconds(100);
}

void PS2MousePos(char &stat, char &x, char &y){
  PS2Write(0xEB);
  PS2Read();
  stat=PS2Read();
  x=PS2Read();
  y=PS2Read();
}


void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');     // send a capital X to indicate that Arduino is running
    delay(300);
  }
}


void setup() {

  digitalWrite(MOTOR_DIRECTION, 0);
  analogWrite(MOTOR_PWM, MAX_DUTYCYCLE);
  delay(100);
  analogWrite(MOTOR_PWM, 0);
  delay(100);

  pinMode(MOTOR_DIRECTION, OUTPUT);
  pinMode(MOTOR_PWM, OUTPUT);

  PS2GoHi(PS2CLOCK);
  PS2GoHi(PS2DATA);

  PS2MouseInit();
  PS2MillisStart=millis();
  MouseX = 0;
  MouseY = 0;

  // start serial port at 115200 bps:
  Serial.begin(115200);
  establishContact();  // send a byte to establish contact until receiver responds
}

void loop() {


  PS2MousePos(stat,x,y);
  MouseX += x; //Set to MouseX -= x to reverse mouse movement
  MouseY += y; //Set to MouseY -= x to reverse mouse movement

//  delay(500);

  millisNow = millis();
  if(millisNow < millisStart){
   millisStart = millisNow;
  }


  dutyCycle = (double)(abs(setPoint - MouseY)) * (double)P_FRACTION;

  if(dutyCycle < MIN_DUTYCYCLE){
    dutyCycle = MIN_DUTYCYCLE;
  }
  if(dutyCycle > MAX_DUTYCYCLE){
    dutyCycle = MAX_DUTYCYCLE;
  }

  if(dutyCycle < MIN_DUTYCYCLE){
    dutyCycle = MIN_DUTYCYCLE;
  }
  if(dutyCycle > MAX_DUTYCYCLE){
    dutyCycle = MAX_DUTYCYCLE;
  }
  if(abs(setPoint - MouseY) < STEP_MARGIN){
    analogWrite(MOTOR_PWM, 0);
    digitalWrite(MOTOR_DIRECTION, 0);
  }
  else{
    if(MouseY < setPoint){
      digitalWrite(MOTOR_DIRECTION, 1);
      analogWrite(MOTOR_PWM, 255 - dutyCycle);
    }
    if(MouseY > setPoint){
      digitalWrite(MOTOR_DIRECTION, 0);
      analogWrite(MOTOR_PWM, dutyCycle);
    }
  }


//  Serial.print("MouseY=");
//  Serial.print(MouseY);
//  Serial.print("\tsetPoint=");
//  Serial.print(setPoint);
//  Serial.println("");

  readByte = 0;
  if (Serial.available() > 0){//if we get a valid byte, read from serial:
    //get incoming byte:
    readByte = Serial.read();
    Serial.print('r');   // send a 'r' to initiate next data from computer

    if(readByte > 0){
      if(readByte == 'h'){
        setPoint++;
      }
      if(readByte == 'g'){
        setPoint--;
      }
      if(readByte == 'f'){
        setPoint+=1000;
      }
      if(readByte == 'e'){
        setPoint-=1000;
      }
      if(readByte == 'd'){
        setPoint+=100;
      }
      if(readByte == 'c'){
        setPoint-=100;
      }
      if(readByte == 'b'){
        setPoint+=10;
      }
      if(readByte == 'a'){
        setPoint-=10;
      }
    }//if(readByte > 0){

  }//while (Serial.available() > 0){

}


