//Project: simple printer
//Homepage: www.HomoFaciens.de
//Author Norbert Heinz
//Version: 0.1
//Creation date: 12.06.2016
//
//Compile with: gcc printer.c -lm -o printer
//Start with: ./commands-host


#include <stdio.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>
#include <unistd.h>
#include <fcntl.h>



#define BAUDRATE B115200
//#define ARDUINOPORT "/dev/ttyUSB0"
#define ARDUINOPORT "/dev/ttyACM0"
#define FALSE 0
#define TRUE 1

#define FEED_PER_LINE 10
#define FEED_PER_DOT  15
#define LINE_REVERSE_OFFSET   0
#define MAX_BITMAP_WIDTH 2000


int fd = 0;
struct termios TermOpt;



//+++++++++++++++++++++++ Start readport ++++++++++++++++++++++++++
char  readport(void){
  int n;
  char buff;
  n = read(fd, &buff, 1);
  if(n > 0){
    return buff;
  }
  return 0;
}
//------------------------ End readport ----------------------------------

//+++++++++++++++++++++++ Start sendport ++++++++++++++++++++++++++
void sendport(unsigned char ValueToSend){
  int n;

  n = write(fd, &ValueToSend, 1);

  if (n < 0){
    printf("write() of value failed!\r");
  }
  else{
    while(readport() != 'r');
  }

}
//------------------------ End sendport ----------------------------------

//+++++++++++++++++++++++ Start openport ++++++++++++++++++++++++++
void openport(void){

    fd = open(ARDUINOPORT, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1)  {
      printf("init_serialport: Unable to open port\n");
    }

    if (tcgetattr(fd, &TermOpt) < 0) {
      printf("init_serialport: Couldn't get term attributes\n");
    }
    speed_t brate = BAUDRATE; // let you override switch below if needed

    cfsetispeed(&TermOpt, brate);
    cfsetospeed(&TermOpt, brate);

    // 8N1
    TermOpt.c_cflag &= ~PARENB;
    TermOpt.c_cflag &= ~CSTOPB;
    TermOpt.c_cflag &= ~CSIZE;
    TermOpt.c_cflag |= CS8;
    // no flow control
    TermOpt.c_cflag &= ~CRTSCTS;

    TermOpt.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    TermOpt.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

    TermOpt.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    TermOpt.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    TermOpt.c_cc[VMIN]  = 0;
    TermOpt.c_cc[VTIME] = 20;

    if( tcsetattr(fd, TCSANOW, &TermOpt) < 0) {
      printf("init_serialport: Couldn't set term attributes\n");
    }

}
//------------------------ End openport ----------------------------------



//######################################################################
//################## Main ##############################################
//######################################################################

int main(int argc, char **argv){

  FILE *FileBMP;

  long i=0, i2, i3, i4;

  char FileInfo[3];
  long FileSize;
  long LongTemp;
  long DataOffset;
  long HeaderSize;
  long PictureWidth;
  long PictureHeight;
  int  IntTemp;
  int  ColorDepth;
  long CompressionType;
  long PictureSize;
  long XPixelPerMeter;
  long YPixelPerMeter;
  long ColorNumber;
  long ColorUsed;
  int PixelRed = 0, PixelGreen = 0, PixelBlue = 0;
  int PixOK=0;
  long PictureWidthRest=0;

  char lineBuffer[MAX_BITMAP_WIDTH];
  int forwardLine = 1;
  long dotsPrinted = 0;


  printf("\n\nOpening port...\n");

  openport();
  printf("...done.\n");

  printf("\n\nWaiting for 'X' from Arduino (Arduino pluged in?)...\n");

  //Wait for 'X' from Arduino
  while(readport() != 'X');




  FileInfo[2]='\0';

  if((FileBMP=fopen("PrintFile.bmp", "rb"))==NULL){
    printf("Can't open PrintFile.bmp!\n");
    return(1);
  }

  fread(&FileInfo, 2, 1, FileBMP);
  fread(&FileSize, 4, 1, FileBMP);
  fread(&LongTemp, 4, 1, FileBMP);
  fread(&DataOffset, 4, 1, FileBMP);
  fread(&HeaderSize, 4, 1, FileBMP);
  fread(&PictureWidth, 4, 1, FileBMP);
  fread(&PictureHeight, 4, 1, FileBMP);
  fread(&IntTemp, 2, 1, FileBMP);
  fread(&ColorDepth, 2, 1, FileBMP);
  fread(&CompressionType, 4, 1, FileBMP);
  fread(&PictureSize, 4, 1, FileBMP);
  fread(&XPixelPerMeter, 4, 1, FileBMP);
  fread(&YPixelPerMeter, 4, 1, FileBMP);
  fread(&ColorNumber, 4, 1, FileBMP);
  fread(&ColorUsed, 4, 1, FileBMP);

  printf("FileInfo=%s\n", FileInfo);
  printf("FileSize=%ld\n", FileSize);
  printf("LongTemp=%ld\n", LongTemp);
  printf("DataOffset=%ld\n", DataOffset);
  printf("HeaderSize=%ld\n", HeaderSize);
  printf("PictureWidth=%ld\n", PictureWidth);
  printf("PictureHeight=%ld\n", PictureHeight);
  printf("IntTemp=%d\n", IntTemp);
  printf("ColorDepth=%d\n", ColorDepth);
  printf("CompressionType=%ld\n", CompressionType);
  printf("PictureSize=%ld\n", PictureSize);
  printf("XPixelPerMeter=%ld\n", XPixelPerMeter);
  printf("YPixelPerMeter=%ld\n", YPixelPerMeter);
  printf("ColorNumber=%ld\n", ColorNumber);
  printf("ColorUsed=%ld\n", ColorUsed);

  if(FileInfo[0] != 'B' || FileInfo[1] != 'M'){
    printf("Wrong Fileinfo (BM)!\n");
    return(1);
  }
  if(ColorDepth != 24){
    printf("Wrong ColorDepth (24)!\n");
    return(1);
  }
  if(PictureWidth > MAX_BITMAP_WIDTH - 1){
    printf("Picture width too large!\n");
    return(1);
  }
  if(CompressionType != 0){
    printf("Wrong CompressionType (0)!\n");
    return(1);
  }


  lineBuffer[PictureWidth] = '\0';

  fseek(FileBMP, DataOffset, SEEK_SET);

  i=0;
  while(i < PictureWidth * 3){
	  i+=4;
  }
  PictureWidthRest = i - PictureWidth * 3;

  printf("PictureWidth=%ld, PictureWidthRest=%ld\n", PictureWidth, PictureWidthRest);

  for(i2 = 0; i2 < PictureHeight; i2++){
    for(i = 0; i < PictureWidth; i++){
      if(fread(&PixelBlue, 1, 1, FileBMP)<1){
	    printf("Reading PixelBlue failed!\n");
	  }
      if(fread(&PixelGreen, 1, 1, FileBMP)<1){
	    printf("Reading PixelGreen failed!\n");
	  }
      if(fread(&PixelRed, 1, 1, FileBMP)<1){
	    printf("Reading PixelRed failed!\n");
	  }
      if(PixelRed > 128 && PixelGreen > 128 && PixelBlue > 128){
        lineBuffer[i] = ' ';
      }
      else{
        lineBuffer[i] = 'X';
        dotsPrinted++;
      }
      //printf("R=% 3d, G=% 3d, B=% d\n", PixelRed, PixelGreen, PixelBlue);
    }//for(i=0;i<PictureWidth;i++){

    if(PictureWidthRest > 0){
		for(i3=0; i3<PictureWidthRest; i3++){
		  if(fread(&PixelBlue, 1, 1, FileBMP)<1){
			printf("Reading PixelBlue failed!\n");
		  }
		}
	}
    for(i4 = 0; i4 < PictureWidth; i4++){
      if(forwardLine == 1){
        if(lineBuffer[i4] == 'X'){
          sendport('R');
        }
        for(i3 = 0; i3 < FEED_PER_DOT; i3++){
		  if(FEED_PER_DOT - i3 >= 15){
		    sendport('k');
		    i3 += 14;
		  }
		  else{
            sendport('i');
		  }
        }
      }
      else{
        for(i3 = 0; i3 < FEED_PER_DOT; i3++){
		  if(FEED_PER_DOT - i3 >= 15){
		    sendport('l');
		    i3 += 14;
		  }
		  else{
            sendport('j');
		  }
        }
        if(lineBuffer[PictureWidth - 1 - i4] == 'X'){
          sendport('R');
        }
      }
    }
    //printf("%s\n", lineBuffer);


    if(forwardLine == 1){
	  for(i3 = 0; i3 < LINE_REVERSE_OFFSET; i3++){
        sendport('j');
	  }
      forwardLine = 0;
    }
    else{
	  for(i3 = 0; i3 < LINE_REVERSE_OFFSET; i3++){
        sendport('i');
	  }
      forwardLine = 1;
    }
    printf("LineFeed No. %ld...", i2);
    for(i3 = 0; i3 < FEED_PER_LINE; i3++){
      sendport('b');
    }
    printf("...done. Dots printed = %ld\n", dotsPrinted);
  }//for(i2=0;i2<PictureHeight;i2++){


  fclose(FileBMP);

  printf("\n...un ferddisch!\n");



  return 0;
}
